from typing import TYPE_CHECKING, Any, Dict

from django.core.mail.message import EmailMultiAlternatives

from rest_registration.notifications.enums import NotificationMethod, NotificationType
from rest_registration.settings import registration_settings
from rest_registration.utils.users import get_user_email_field_name

if TYPE_CHECKING:
    from django.contrib.auth.base_user import AbstractBaseUser


def send_verification_notification(
        notification_type: NotificationType,
        user: 'AbstractBaseUser',
        data: Dict[str, Any],
        template_config_data: Dict[str, Any],
        custom_user_address: Any = None) -> None:
    if custom_user_address is None:
        user_address = get_user_address(user)
    else:
        user_address = custom_user_address
    notification = create_verification_notification(
        notification_type, user, user_address, data, template_config_data)
    send_notification(notification)


def create_verification_notification(
        notification_type: NotificationType,
        user: 'AbstractBaseUser',
        user_address: Any,
        data: Dict[str, Any],
        template_config_data: Dict[str, Any]) -> EmailMultiAlternatives:
    from_email = registration_settings.VERIFICATION_FROM_EMAIL
    reply_to_email = (registration_settings.VERIFICATION_REPLY_TO_EMAIL or
                      from_email)
    template_context_builder = registration_settings.VERIFICATION_TEMPLATE_CONTEXT_BUILDER  # noqa: E501
    context = template_context_builder(
        user, user_address, data,
        notification_type=notification_type,
        notification_method=NotificationMethod.EMAIL)

    template_builder = registration_settings.VERIFICATION_TEMPLATE_RENDERER  # noqa: E501
    template_data = template_builder(template_config_data, context)

    email_msg = EmailMultiAlternatives(
        subject=template_data.subject,
        body=template_data.text_body,
        from_email=from_email,
        to=[user_address],
        reply_to=[reply_to_email]
    )
    if template_data.html_body:
        email_msg.attach_alternative(template_data.html_body, 'text/html')

    return email_msg


def send_notification(notification: EmailMultiAlternatives) -> None:
    notification.send()


def get_user_address(user: 'AbstractBaseUser') -> str:
    email_field_name = get_user_email_field_name()
    email = getattr(user, email_field_name)  # type: str
    return email



from django.urls import path

from .views import (
    change_password,
    login,
    logout,
    profile,
    register,
    register_email,
    reset_password,
    send_reset_password_link,
    verify_email,
    verify_registration
)

app_name = 'rest_registration'
urlpatterns = [
    path('register/', register, name='register'),
    path('verify-registration/', verify_registration, name='verify-registration'),

    path(
        'send-reset-password-link/', send_reset_password_link,
        name='send-reset-password-link',
    ),
    path('reset-password/', reset_password, name='reset-password'),

    path('login/', login, name='login'),
    path('logout/', logout, name='logout'),

    path('profile/', profile, name='profile'),

    path('change-password/', change_password, name='change-password'),

    path('register-email/', register_email, name='register-email'),
    path('verify-email/', verify_email, name='verify-email'),
]



#!/usr/bin/env python
import os.path
import re

from setuptools import find_packages, setup

ROOT_DIR = os.path.dirname(__file__)
PACKAGE_NAME = 'rest_registration'


def get_version(package):
    '''
    Return package version as listed in `__version__` in package `__init__.py`.
    '''
    init_path = os.path.join(ROOT_DIR, package, '__init__.py')
    with open(init_path, 'rt', encoding='utf-8') as init_file:
        init_contents = init_file.read()
        return re.search(
            "__version__ = ['\"]([^'\"]+)['\"]", init_contents).group(1)


setup(
    version=get_version(PACKAGE_NAME),
    packages=find_packages(exclude=['tests.*', 'tests']),
)



from typing import Any, Dict, Iterable, Optional

from django.conf import settings as root_settings
from rest_framework.settings import perform_import


class NestedSettings:
    def __init__(
            self,
            user_settings: Optional[Dict[str, Any]],
            defaults: Dict[str, Any],
            import_strings: Iterable[str],
            root_setting_name: str) -> None:
        if user_settings:
            self._user_settings = user_settings
        self.defaults = defaults
        self.import_strings = import_strings
        self.root_setting_name = root_setting_name

    @property
    def user_settings(self) -> Dict[str, Any]:
        if not hasattr(self, '_user_settings'):
            self._user_settings = getattr(
                root_settings,
                self.root_setting_name,
                {},
            )
        return self._user_settings

    def reset_user_settings(self) -> None:
        if hasattr(self, '_user_settings'):
            del self._user_settings

    def reset_attr_cache(self) -> None:
        for key in self.defaults.keys():
            if hasattr(self, key):
                delattr(self, key)

    def is_default(self, attr: str) -> bool:
        if attr not in self.user_settings:
            return True
        return self.user_settings[attr] == self.defaults[attr]

    def __getattr__(self, attr: str) -> Any:
        if attr not in self.defaults.keys():
            raise AttributeError(
                "Invalid {self.root_setting_name} setting: '{attr}'".format(
                    self=self, attr=attr))

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if attr in self.import_strings:
            val = perform_import(val, attr)

        # Cache the result
        setattr(self, attr, val)
        return val
